==> tag_1.0.0.zip: server/src/main/java/cn/com/qmth/scancloud/service/impl/PaperPageServiceImpl.java <==
package cn.com.qmth.scancloud.service.impl;

import cn.com.qmth.scancloud.bean.User;
import cn.com.qmth.scancloud.bean.omredit.OmrEditDomain;
import cn.com.qmth.scancloud.bean.omredit.OmrEditPage;
import cn.com.qmth.scancloud.bean.omredit.OmrEditPaper;
import cn.com.qmth.scancloud.bean.papermigrate.PaperMigrateDomain;
import cn.com.qmth.scancloud.bean.papermigrate.PaperMigratePage;
import cn.com.qmth.scancloud.dao.PaperPageDao;
import cn.com.qmth.scancloud.entity.*;
import cn.com.qmth.scancloud.enums.LockType;
import cn.com.qmth.scancloud.enums.ScanStatus;
import cn.com.qmth.scancloud.enums.UploadStatus;
import cn.com.qmth.scancloud.service.*;
import cn.com.qmth.scancloud.vo.PaperMigrateVo;
import cn.com.qmth.scancloud.vo.UpdateTimeVo;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.jeffreyning.mybatisplus.service.MppServiceImpl;
import com.qmth.boot.core.concurrent.service.ConcurrentService;
import com.qmth.boot.core.exception.ParameterException;
import com.qmth.boot.core.exception.StatusException;
import com.qmth.boot.core.fss.store.FileStore;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;

@Service
public class PaperPageServiceImpl extends MppServiceImpl<PaperPageDao, PaperPageEntity> implements PaperPageService {

    @Autowired
    private OmrTaskService omrTaskService;

    @Autowired
    private AnswerCardService answerCardService;

    @Autowired
    private StudentService studentService;

    @Autowired
    private StudentPaperService studentPaperService;

    @Autowired
    private ConcurrentService concurrentService;

    @Autowired
    private PaperService paperService;

    @Autowired
    private FileStore fileStore;

    @Override
    public List<PaperPageEntity> listByPaperId(Long paperId) {
        if (paperId == null) {
            throw new ParameterException("paperId不能为空");
        }
        QueryWrapper<PaperPageEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<PaperPageEntity> lw = wrapper.lambda();
        lw.eq(PaperPageEntity::getPaperId, paperId);
        wrapper.orderByAsc("page_index");
        return this.list(wrapper);
    }

    @Transactional
    @Override
    public void deleteByBatchIdAndStudentId(Long batchId, Long studentId) {
        if (batchId == null) {
            throw new ParameterException("批次ID不能为空");
        }
        if (studentId == null) {
            throw new ParameterException("考生ID不能为空");
        }
        QueryWrapper<PaperPageEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<PaperPageEntity> lw = wrapper.lambda();
        lw.eq(PaperPageEntity::getBatchId, batchId);
        lw.eq(PaperPageEntity::getStudentId, studentId);
        this.baseMapper.delete(wrapper);
    }

    @Override
    public PaperPageEntity findPaperIdAndIndex(Long paperId, Integer index) {
        if (paperId == null) {
            throw new ParameterException("paperId不能为空");
        }
        if (index == null) {
            throw new ParameterException("index不能为空");
        }
        QueryWrapper<PaperPageEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<PaperPageEntity> lw = wrapper.lambda();
        lw.eq(PaperPageEntity::getPaperId, paperId);
        lw.eq(PaperPageEntity::getPageIndex, index);
        return baseMapper.selectOne(wrapper);
    }

    @Override
    public Integer getUploadSheetCount(Long paperId) {
        QueryWrapper<PaperPageEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<PaperPageEntity> lw = wrapper.lambda();
        lw.eq(PaperPageEntity::getPaperId, paperId);
        lw.ne(PaperPageEntity::getSheetPath, null);
        return this.count(wrapper);
    }

    @Override
    public List<PaperPageEntity> findByBatch(Long id) {
        QueryWrapper<PaperPageEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<PaperPageEntity> lw = wrapper.lambda();
        lw.eq(PaperPageEntity::getBatchId, id);
        wrapper.orderByAsc("page_index");
        return this.list(wrapper);
    }

    @Override
    public List<PaperPageEntity> listByPaperList(List<Long> paperIds) {
        QueryWrapper<PaperPageEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<PaperPageEntity> lw = wrapper.lambda();
        lw.in(PaperPageEntity::getPaperId, paperIds);
        wrapper.orderByAsc("page_index");
        return this.list(wrapper);
    }

    @Transactional
    @Override
    public UpdateTimeVo omrEdit(OmrEditDomain domain) {
        Long examId = domain.getExamId();
        if (examId == null) {
            throw new ParameterException("考试ID不能为空");
        }
        String examNumber = domain.getExamNumber();
        if (StringUtils.isBlank(examNumber)) {
            throw new ParameterException("准考证号不能为空");
        }
        StudentEntity student = studentService.findByExamAndExamNumber(examId, examNumber);
        if (student == null) {
            throw new ParameterException("考生信息未找到");
        }
        if (CollectionUtils.isEmpty(domain.getPapers())) {
            throw new ParameterException("papers信息不能为空");
        }
        concurrentService.getReadWriteLock(LockType.STUDENT + "-" + student.getId()).writeLock().lock();
        try {
            boolean studentQuestionFilled = false;
            for (OmrEditPaper paper : domain.getPapers()) {
                Integer number = paper.getNumber();
                if (number == null) {
                    throw new ParameterException("Paper number不能为空");
                }
                StudentPaperEntity sp = studentPaperService.findByStudentIdAndNumber(student.getId(), number);
                if (sp == null) {
                    throw new ParameterException("未找到绑定扫描结果");
                }
                PaperEntity oldpaper = paperService.getById(sp.getPaperId());
                if (CollectionUtils.isEmpty(paper.getPages())) {
                    throw new ParameterException("pages信息不能为空");
                }
                boolean paperQuestionFilled = false;
                for (OmrEditPage page : paper.getPages()) {
                    Integer index = page.getIndex();
                    if (index == null) {
                        throw new ParameterException("Page index不能为空");
                    }
                    PaperPageEntity pageSave = findPaperIdAndIndex(sp.getPaperId(), index);
                    if (pageSave == null) {
                        throw new ParameterException("未找扫描结果");
                    }
                    if (number == 1 && index == 1) {
                        if (page.getPaperType() != null) {
                            student.setPaperType( StringUtils.isBlank(page.getPaperType().getResult())
                                    ? "#": page.getPaperType().getResult());
                        }
                        if (page.getAbsent() != null) {
                            student.setOmrAbsent(page.getAbsent().getResult());
                        }
                    }
                    if (page.getAbsent() != null) {
                        pageSave.setAbsent(page.getAbsent());
                    }
                    if (page.getBreach() != null) {
                        pageSave.setBreach(page.getBreach());
                    }
                    if (page.getPaperType() != null) {
                        pageSave.setPaperType(page.getPaperType());
                    }
                    if (page.getQuestion() != null) {
                        pageSave.setQuestion(page.getQuestion());
                    }
                    if (page.getSelective() != null) {
                        pageSave.setSelective(page.getSelective());
                    }
                    if (page.getQuestion() != null && checkQuestionFilled(page.getQuestion().getResult())) {
                        paperQuestionFilled = true;
                        studentQuestionFilled = true;
                    }
                    this.saveOrUpdateByMultiId(pageSave);
                }
                oldpaper.setQuestionFilled(paperQuestionFilled);
                paperService.saveOrUpdate(oldpaper);
            }
            student.setQuestionFilled(studentQuestionFilled);
            student.setDataUploadStatus(UploadStatus.WAITING_UPLOAD);
            studentService.saveOrUpdate(student);
            UpdateTimeVo vo = new UpdateTimeVo();
            vo.setUpdateTime(new Date().getTime());
            return vo;
        } finally {
            concurrentService.getReadWriteLock(LockType.STUDENT + "-" + student.getId()).writeLock().unlock();
        }
    }

    private boolean checkQuestionFilled(List<String> list) {
        if (CollectionUtils.isEmpty(list)) {
            return false;
        }
        for (String s : list) {
            if (!"#".equals(s)) {
                return true;
            }
        }
        return false;
    }

    @Transactional
    @Override
    public PaperMigrateVo paperMigrate(User user, PaperMigrateDomain domain) {
        Long examId = domain.getExamId();
        if (examId == null) {
            throw new ParameterException("考试ID不能为空");
        }
        String examNumber = domain.getExamNumber();
        if (StringUtils.isBlank(examNumber)) {
            throw new ParameterException("准考证号不能为空");
        }
        StudentEntity student = studentService.findByExamAndExamNumber(examId, examNumber);
        if (student == null) {
            throw new ParameterException("考生信息未找到");
        }
        concurrentService.getReadWriteLock(LockType.STUDENT + "-" + student.getId()).writeLock().lock();
        StudentEntity oldStudent = null;
        try {
            // 解绑原paper和其考生
            StudentPaperEntity oldSp = studentPaperService.findByPaperId(domain.getPaperId());
            Integer paperNumber = null;
            if (oldSp != null) {
                paperNumber =oldSp.getPaperNumber();
                if (!oldSp.getStudentId().equals(student.getId())) {// 非同一个考生
                    oldStudent = studentService.getById(oldSp.getStudentId());
                    if (oldStudent == null) {
                        throw new ParameterException("源考生信息未找到");
                    }
                    concurrentService.getReadWriteLock(LockType.STUDENT + "-" + oldSp.getStudentId()).writeLock()
                            .lock();
                }
                studentPaperService.deleteByMultiId(oldSp);
            }
            if (domain.getPaperNumber() == null) {
                throw new ParameterException("PaperNumber不能为空");
            }
            if (domain.getCardNumber() == null) {
                throw new ParameterException("CardNumber不能为空");
            }
            if (student.getCardNumber() != null && !student.getCardNumber().equals(domain.getCardNumber())) {
                throw new ParameterException("CardNumber与考生不一致");
            }
            Long paperId = domain.getPaperId();
            if (paperId == null) {
                throw new ParameterException("paperId不能为空");
            }
            PaperEntity paper = paperService.getById(paperId);
            if (paper == null) {
                throw new ParameterException("paper信息未找到");
            }

            if (student.getCardNumber()!=null && !paper.getCardNumber().equals(domain.getCardNumber()) && CollectionUtils.isEmpty(domain.getPages())) {
                throw new ParameterException("卡格式有变化需要重新识别,pages信息不能为空");
            }
            paper.setCardNumber(domain.getCardNumber());
            if (paperNumber!=null && !paperNumber.equals(domain.getPaperNumber()) && CollectionUtils.isEmpty(domain.getPages())) {
                throw new ParameterException("paperNumber有变化需要重新识别,pages信息不能为空");
            }
            // paper.setNumber(domain.getPaperNumber());
            AnswerCardEntity answerCard = answerCardService.findByExamAndNumber(paper.getExamId(),
                    domain.getCardNumber());
            if (answerCard == null) {
                throw new ParameterException("卡格式信息未找到");
            }
            if (StringUtils.isNotBlank(answerCard.getSubjectCode())
                    && !answerCard.getSubjectCode().equals(student.getSubjectCode())) {
                throw new ParameterException("卡格式科目代码不一致");
            }
            if (CollectionUtils.isNotEmpty(domain.getPages())) {
                if (answerCard.getSinglePage() && domain.getPages().size() != 1) {
                    throw new ParameterException("卡格式页数不一致");
                }
                if (!answerCard.getSinglePage() && domain.getPages().size() != 2) {
                    throw new ParameterException("卡格式页数不一致");
                }
                for (PaperMigratePage page : domain.getPages()) {
                    if (CollectionUtils.isNotEmpty(page.getSliceUri())) {
                        for (String spath : page.getSliceUri()) {
                            if (!fileStore.exist(spath)) {
                                throw new StatusException("图片不存在:" + spath);
                            }
                        }
                    }
                    Integer index = page.getIndex();
                    if (index == null) {
                        throw new ParameterException("Page index不能为空");
                    }
                    PaperPageEntity pageSave = findPaperIdAndIndex(paperId, index);
                    if (pageSave == null) {
                        throw new ParameterException("未找到page信息 pageIndex:" + index);
                    }
                    pageSave.setSlicePath(page.getSliceUri());
                    pageSave.setAbsent(page.getAbsent());
                    pageSave.setBreach(page.getBreach());
                    pageSave.setPaperType(page.getPaperType());
                    pageSave.setQuestion(page.getQuestion());
                    pageSave.setSelective(page.getSelective());
                    pageSave.setRecogData(page.getRecogData());
                    this.saveOrUpdateByMultiId(pageSave);
                }
            }
            // 修改的paper强制置为人工绑定
            paper.setAssigned(true);
            // 原有mismatch标记需要清除
            paper.setMismatch(false);
            paperService.saveOrUpdate(paper);

            // 查找考生现有绑定paper
            StudentPaperEntity sp = studentPaperService.findByStudentIdAndNumber(student.getId(),
                    domain.getPaperNumber());
            if (sp != null) {
                if (sp.getPaperId().equals(domain.getPaperId())) {
                    throw new ParameterException("该题卡已绑定该考生");
                }
                // 现有绑定paper强制置为mismatch
                PaperEntity oldpaper = paperService.getById(sp.getPaperId());
                oldpaper.setMismatch(true);
                paperService.saveOrUpdate(oldpaper);
                sp.setPaperId(paperId);
            } else {
                sp = new StudentPaperEntity();
                sp.setPaperId(paperId);
                sp.setStudentId(student.getId());
                sp.setPaperNumber(domain.getPaperNumber());
            }
            studentPaperService.saveOrUpdateByMultiId(sp);
            // 同步更新考生状态
            disposeStudent(student, oldStudent, answerCard.getPaperCount());
            // 更新识别对照任务
            if (oldStudent != null) {
                omrTaskService.rebuild(user, examId, oldStudent.getId());
            }
            omrTaskService.rebuild(user, examId, student.getId());
            // 返回数据结果
            PaperMigrateVo vo = new PaperMigrateVo();
            vo.setPaperId(domain.getPaperId());
            vo.setUpdateTime(paper.getUpdateTime());
            return vo;
        } finally {
            concurrentService.getReadWriteLock(LockType.STUDENT + "-" + student.getId()).writeLock().unlock();
            if (oldStudent != null) {
                concurrentService.getReadWriteLock(LockType.STUDENT + "-" + oldStudent.getId()).writeLock().unlock();
            }
        }
    }

    private void disposeStudent(StudentEntity target, StudentEntity source, int paperCount) {
        target.setAssigned(true);
        updateStudentStatus(target, paperCount);
        if (source != null) {
            updateStudentStatus(source, paperCount);
        }
    }

    private void updateStudentStatus(StudentEntity s, int paperCount) {
        List<PaperEntity> ps = paperService.findStudentPaperEffective(s.getId());
        if (CollectionUtils.isEmpty(ps)) {
            s.setIncomplete(false);
            s.setAssigned(false);
            s.setOmrAbsent(false);
            s.setCardNumber(null);
            s.setPaperType("#");
            s.setQuestionFilled(false);
            s.setStatus(ScanStatus.UNEXIST);
        } else {
            s.setIncomplete(ps.size() < paperCount);
            s.setStatus(ScanStatus.SCANNED);
            Integer cardNumber =s.getCardNumber();
            boolean questionFilled = false;
            PaperEntity firstPaper = null;
            StudentPaperEntity sp = studentPaperService.findByStudentIdAndNumber(s.getId(),1);
            if(sp !=null) {
                firstPaper = paperService.getById(sp.getPaperId());
            }
            for (PaperEntity p : ps) {
                if(cardNumber ==null) {
                    cardNumber = p.getCardNumber();
                }
                if (p.getQuestionFilled() != null && p.getQuestionFilled()) {
                    questionFilled = true;
                    break;
                }
            }
            s.setQuestionFilled(questionFilled);
            s.setCardNumber(cardNumber);
            if (firstPaper != null) {
                PaperPageEntity firstPage = this.findPaperIdAndIndex(firstPaper.getId(), 1);
                if (firstPage == null) {
                    throw new ParameterException("未找到page信息 papaerId:" + firstPaper.getId() + " papgeIndex:1");
                }
                s.setPaperType(
                        firstPage.getPaperType() == null || StringUtils.isBlank(firstPage.getPaperType().getResult())
                                ? "#"
                                : firstPage.getPaperType().getResult());
                s.setOmrAbsent(firstPage.getAbsent() == null ? false : firstPage.getAbsent().getResult());
            }else {
                s.setOmrAbsent(false);
                s.setPaperType("#");
            }
        }
        s.setFileUploadStatus(UploadStatus.WAITING_UPLOAD);
        s.setDataUploadStatus(UploadStatus.WAITING_UPLOAD);
        studentService.saveOrUpdate(s);
    }

}

==> tag_1.0.2.zip: server/src/main/java/cn/com/qmth/scancloud/service/impl/PaperPageServiceImpl.java <==
package cn.com.qmth.scancloud.service.impl;

import java.util.List;

import javax.validation.constraints.NotNull;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.github.jeffreyning.mybatisplus.service.MppServiceImpl;
import com.qmth.boot.core.exception.ParameterException;

import cn.com.qmth.scancloud.dao.PaperPageDao;
import cn.com.qmth.scancloud.entity.PaperPageEntity;
import cn.com.qmth.scancloud.service.PaperPageService;
import cn.com.qmth.scancloud.vo.paper.PaperPageCetVo;

@Service
public class PaperPageServiceImpl extends MppServiceImpl<PaperPageDao, PaperPageEntity> implements PaperPageService {

    @Override
    @Transactional
    public void deleteByPaperId(Long paperId) {
        QueryWrapper<PaperPageEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<PaperPageEntity> lw = wrapper.lambda();
        lw.eq(PaperPageEntity::getPaperId, paperId);
        baseMapper.delete(wrapper);
    }

    @Override
    @Transactional
    public boolean updateSlicePath(@NotNull PaperPageEntity page) {
        LambdaUpdateWrapper<PaperPageEntity> lw = new LambdaUpdateWrapper<>();
        lw.set(PaperPageEntity::getSlicePath, page.getSlicePath());
        lw.eq(PaperPageEntity::getPaperId, page.getPaperId());
        lw.eq(PaperPageEntity::getPageIndex, page.getPageIndex());
        return update(lw);
    }
    
    @Override
    @Transactional
    public boolean updateSheetPath(@NotNull PaperPageEntity page) {
        LambdaUpdateWrapper<PaperPageEntity> lw = new LambdaUpdateWrapper<>();
        lw.set(PaperPageEntity::getSheetPath, page.getSheetPath());
        lw.eq(PaperPageEntity::getPaperId, page.getPaperId());
        lw.eq(PaperPageEntity::getPageIndex, page.getPageIndex());
        return update(lw);
    }

    @Override
    public List<PaperPageEntity> listByPaperId(Long paperId) {
        if (paperId == null) {
            throw new ParameterException("paperId不能为空");
        }
        QueryWrapper<PaperPageEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<PaperPageEntity> lw = wrapper.lambda();
        lw.eq(PaperPageEntity::getPaperId, paperId);
        wrapper.orderByAsc("page_index");
        return this.list(wrapper);
    }

    @Override
    public PaperPageEntity findPaperIdAndIndex(Long paperId, Integer index) {
        if (paperId == null) {
            throw new ParameterException("paperId不能为空");
        }
        if (index == null) {
            throw new ParameterException("index不能为空");
        }
        QueryWrapper<PaperPageEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<PaperPageEntity> lw = wrapper.lambda();
        lw.eq(PaperPageEntity::getPaperId, paperId);
        lw.eq(PaperPageEntity::getPageIndex, index);
        return baseMapper.selectOne(wrapper);
    }

    @Override
    public List<PaperPageEntity> listByPaperList(List<Long> paperIds) {
        QueryWrapper<PaperPageEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<PaperPageEntity> lw = wrapper.lambda();
        lw.in(PaperPageEntity::getPaperId, paperIds);
        wrapper.orderByAsc("paper_id", "page_index");
        return this.list(wrapper);
    }


    @Override
    public List<PaperPageCetVo> listByStudentIds(List<Long> studentIds) {
        return baseMapper.listByStudentIds(studentIds);
    }
}
